import sys
import os

from nose.tools import with_setup, assert_equals

from atomisator.main.run import atomisator, _parse_options
from atomisator.main.run import CONF_TMPL, _get_reader 
from atomisator.main.config import AtomisatorConfig
from atomisator.main.run import generate_config 
from atomisator.main.run import load_feeds, generate_feed
from atomisator.main.run import _enhancers, _filters

saved = None

def setup():
    global saved
    saved = sys.argv

def teardown():
    sys.argv = saved

@with_setup(setup, teardown)
def test_options():
    # making sure we know how to parse options
    sys.argv = ['atomisator']
    options = _parse_options()
    assert_equals(options.read, True)
    assert_equals(options.generate, True)
    assert_equals(options.create, None)
    assert_equals(options.config, 'atomisator.cfg')

    sys.argv = ['atomisator', '-c', 'myfile.cfg']
    options = _parse_options()
    assert_equals(options.create, 'myfile.cfg')
    assert_equals(options.read, False)
    assert_equals(options.generate, False)

    sys.argv = ['atomisator', '-r', '-g', '-f', 'here.cfg']
    options = _parse_options()
    assert_equals(options.create, None)
    assert_equals(options.read, True)
    assert_equals(options.generate, True)
    assert_equals(options.config, 'here.cfg')

    sys.argv = ['atomisator', 'here.cfg']
    options = _parse_options()
    assert_equals(options.create, None)
    assert_equals(options.read, True)
    assert_equals(options.generate, True)
    assert_equals(options.config, 'here.cfg')

    sys.argv = ['atomisator', 'here.cfg', '-g']
    options = _parse_options()
    assert_equals(options.create, None)
    assert_equals(options.read, False)
    assert_equals(options.generate, True)
    assert_equals(options.config, 'here.cfg')

def gen_init():
    if os.path.exists('here.cfg'):
        os.remove('here.cfg')

@with_setup(setup=gen_init, teardown=gen_init)
def test_generation():
    # Let's try a generation
    generate_config('here.cfg')
    assert_equals(open('here.cfg').read(), CONF_TMPL)

test_dir = os.path.dirname(__file__)
package_dir = os.path.split(test_dir)[0]
test_conf = os.path.join(test_dir, 'atomisator.cfg')

def set_conf():
    template = open(os.path.join(test_dir, 'atomisator.cfg_tmpl')).read()
    cfg = template % {'test_dir': test_dir}
    f = open(test_conf, 'w')
    f.write(cfg)
    f.close()

def tear_conf():
    if os.path.exists(test_conf):
        os.remove(test_conf)
    xml = os.path.join(test_dir, 'atomisator.xml')
    if os.path.exists(xml):
        os.remove(xml)

@with_setup(set_conf, tear_conf)
def test_load_feeds():
    load_feeds(test_conf)

@with_setup(set_conf, tear_conf)
def test_generate_feed():
    generate_feed(test_conf)

@with_setup(set_conf, tear_conf)
def test_config():
    parser = AtomisatorConfig(test_conf)
    sources = parser.sources
    sources.sort()
    dir = os.path.dirname(test_conf)
    wanted = [('rss', ('%s/digg.xml' % dir,)), 
              ('rss', ('%s/digg.xml' % dir,)), 
              ('rss', ('%s/pp.xml' % dir,)), 
              ('rss', ('%s/pp.xml' % dir,)), 
              ('rss', ('%s/tarek.xml' % dir,)),
              ('rss', ('%s/tarek.xml' % dir,))]


    assert_equals(sources, wanted)
    wanted = os.path.join(dir, 'atomisator.db')
    assert_equals(parser.database, 'sqlite:///%s' % wanted)

    # getting readers
    assert_equals(parser.get_reader('xxx'), None)
    assert_equals(parser.get_reader('xml'), 
                  'atomisator.reader.xml')

    f = [f[0] for f in parser.filters]
    f.sort()
    assert_equals(f, ['autotags', 'buzzwords', 'doublons', 'spam', 'stopwords'])

def test_get_readers():
    # see if we get the rss and atom plugin
    from atomisator.parser import Parser
    
    assert_equals(_get_reader('xxxx'), None)
    assert_equals(_get_reader('rss'), Parser)

def test_filters():
    assert 'doublons' in _filters

def test_enhancers():
    assert 'digg' in _enhancers




import os
import sys
from os.path import join
from ConfigParser import ConfigParser

DEFAULT_CONFIG_FILE = 'atomisator.cfg'

class AtomisatorConfig(ConfigParser):
    def __init__(self, cfg=None):
        ConfigParser.__init__(self)
        if cfg is None:
            cfg = DEFAULT_CONFIG_FILE
        if not os.path.exists(cfg):
            raise ValueError('Could not read configuration (%s)' % cfg)
        self.read([cfg])

    def get_reader(self, name):
        if not self.has_section('readers'):
            return None
        if not self.has_option('readers', name):
            return None
        return self.get('readers', name)

    def _get_sources(self):
        sources = self.get('atomisator', 'sources').split('\n')
        def _args(p):
            p = p.split()
            return p[0].strip(), tuple([p.strip() 
                                        for p in p[1:]])
        return [_args(s) for s in sources if s.strip() != '']
    sources = property(_get_sources)

    def _get_filters(self):
        if not self.has_option('atomisator', 'filters'):
            return []
        filters = self.get('atomisator', 'filters').split('\n')
        def _args(p):
            p = p.split()
            return p[0].strip(), tuple([p.strip() 
                                        for p in p[1:]])
        return [_args(s) for s in filters if s.strip() != '']
    filters = property(_get_filters)

    def _get_enhancers(self):
        if not self.has_option('atomisator', 'enhancers'):
            return []
        enhancers = self.get('atomisator', 'enhancers').split('\n')
        def _args(p):
            p = p.split()
            return p[0].strip(), tuple([p.strip() 
                                        for p in p[1:]])
        return [_args(e) for e in enhancers if e.strip() != '']
    enhancers = property(_get_enhancers)

    def _get_simple_field(self, field):
        return self.get('atomisator', field).strip()

    def _get_database(self):
        return self._get_simple_field('database')
    database = property(_get_database)

    def _get_title(self):
        return self._get_simple_field('title')
    title = property(_get_title)

    def _get_description(self):
        return self._get_simple_field('description')
    description = property(_get_description)

    def _get_link(self):
        return self._get_simple_field('link')
    link = property(_get_link)

    def _get_file(self):
        return self._get_simple_field('file')
    file = property(_get_file)




import sys
import os

from optparse import OptionParser
from optparse import OptionValueError
from setuptools.package_index import iter_entry_points

from atomisator.main.config import AtomisatorConfig
from atomisator.main import __version__ as VERSION

from atomisator.db.session import create_session
from atomisator.db.core import create_entry
from atomisator.db.core import get_entries
from atomisator.feed import generate
from atomisator.db.session import commit

def _log(msg):
    print msg

CONF_TMPL = """\
[atomisator]

# put here the sources you wish to process
# the first parameter is the type of source
# and the following parameters are the arguments
# passed to the reader
sources = 
    rss http://tarekziade.wordpress.com/atom 
    rss http://digg.com/rss/index.xml

# put here the filters you want to use
filters =
    doublons

# put here the enhancers you want to use
enhancers =

# put here the database location
database = sqlite:///atomisator.db

# this is the filename that will be generated
file = atomisator.xml

# infos that will appear in the generated feed. 
title = meta
description = Automatic feed created by Atomisator.
link =  http://atomisator.ziade.org/example
"""

def generate_config(path):
    """creates a default config file"""
    if os.path.exists(path):
        raise ValueError('%s already exists.' % path)
    f = open(path, 'w')
    try:
        f.write(CONF_TMPL)
    finally:
        f.close()
    _log('Default config generated at "%s."' % path)


def _get_reader(name):
    readers = list(iter_entry_points('atomisator.readers', name))
    if len(readers) == 0:
        # not found, let's try to get it
        # not implemented yet
        return None
    return readers[0].load()

_fs = iter_entry_points('atomisator.filters')
_filters = dict([(f.name, f.load()()) for f in _fs])

def _apply_filters(entry, entries, filters):
    for f, args in filters:
        entry = f(entry, entries, *args)
        if entry is None:
            return None
    return entry

def load_feeds(conf):
    """Fetches feeds."""
    parser = AtomisatorConfig(conf)
    create_session(parser.database)
    count = 0

    # initial entries, see if this call is optimal
    existing_entries = get_entries().all()

    filter_chain = set([(_filters[name], args) 
                        for name, args in parser.filters 
                        if name in _filters])

    for reader, args in parser.sources:
        # check if the readers is available
        pl = _get_reader(reader)

        if pl is None:
            raise ValueError('%s reader not found' % pl) 
            
        _log('Reading source %s' % ' '.join(args))
        scount = 0

        for entry in pl()(*args):
            entry = _apply_filters(entry, existing_entries, filter_chain)
            if entry is None:
                continue
            id_, new_entry = create_entry(entry, commit=False)
            count += 1
            scount += 1
            existing_entries.append(new_entry)
        _log('%d entries read.' % scount)
    _log('%d total.' % count)
    commit()    # final commit

_es = iter_entry_points('atomisator.enhancers')
_enhancers = dict([(e.name, e.load()()) for e in _es])

def _select_enhancers(enhancers):
    res = []
    for name, args in enhancers:
        if name in _enhancers:
            res.append((_enhancers[name], args))
    return res

def generate_feed(conf):
    """Creates the meta-feed."""
    if conf is None:
        conf = _get_opt()
    parser = AtomisatorConfig(conf)
    create_session(parser.database)
    _log('Writing feed in %s' % parser.file)
    enhancers = _select_enhancers(parser.enhancers)
    feed = generate(parser.title, parser.description, parser.link, enhancers)
    
    f = open(parser.file, 'w')
    try:
        f.write(feed)
    finally:
        f.close()
    _log('Feed ready.')

def _parse_options():
    """Calling both."""
    parser = OptionParser(usage='usage: %prog [options]',
                          version='%%prog %s' % VERSION)

    parser.add_option("-c", "--create-config", dest="create",
                      action="store",
                      help="Creates a default config file.",
                      metavar="CONFIG_FILE") 

    parser.add_option("-r", "--read", dest="read",
                      action="store_true",
                      help="Reads sources.", default=False)
 
    parser.add_option("-l", "--list-filters", dest="filters",
                      action="store_true",
                      help="List all filters.", default=False)
   
    parser.add_option("-p", "--list-readers", dest="readers",
                      action="store_true",
                      help="List all readers.", default=False)

    parser.add_option("-e", "--list-enhancers", dest="enhancers",
                      action="store_true",
                      help="List all enhancers.", default=False)

    parser.add_option("-g", "--generate", dest="generate",
                      action="store_true",
                      help="Generates feed.", default=False)
        
    parser.add_option("-f", "--config-file", dest="config",
                      help="Points to the configuration file.",
                      metavar="CONFIG_FILE") 
    
    options, args = parser.parse_args()

    if options.create is not None:
        options.read = False
        options.generate = False
    else:
        if options.config is None:
            if len(args) > 0:
                options.config = args[0]
            else:
                options.config = 'atomisator.cfg'

        if options.read is False and options.generate is False:
            options.read = True
            options.generate = True

    return options

def list_readers():
    for p in iter_entry_points('atomisator.readers'):
        print p.name
        d = p.load().__doc__
        if d is not None:
            print d
            
def list_filters():
    for key, ob in _filters.items():
        print '%s: %s' % (key, ob.__doc__)

def list_enhancers():
    for key, ob in _enhancers.items():
        print '%s: %s' % (key, ob.__doc__)

def atomisator():
    options = _parse_options()
    if options.filters:
        list_filters()
        sys.exit(0)

    if options.readers:
        list_readers()
        sys.exit(0)

    if options.enhancers:
        list_enhancers()
        sys.exit(0)

    if options.create is not None:
        generate_config(options.create)
        sys.exit(0)
    
    if options.read:
        load_feeds(options.config)
    if options.generate:
        generate_feed(options.config)

